from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('location.views',
        url(r'^foursquare/', 'foursquare_checkin', name='foursquare_push'),
        url(r'^kml/', 'get_kml', ),
        url(r'^$', 'my_location', ),
    )



import logging

from django import template
from django.conf import settings
from django.contrib.gis.geos import Point

from location.models import LocationSnapshot

register = template.Library()

logger = logging.getLogger('location.templatetags.current_location')

LOCATION_HOME = getattr(settings, 'LOCATION_HOME', None)

class LocationSnapshotNode(template.Node):
    def __init__(self, variable, username):
        self.username = username
        self.variable = variable

    def render(self, context):
        try:
            snapshot_query = LocationSnapshot.objects.filter(
                        user__username = self.username
                    ).order_by('-date')
            if LOCATION_HOME:
                snapshot_query = snapshot_query.distance(
                        Point(
                            *LOCATION_HOME
                            )
                        )
            snapshot = snapshot_query[0]
            logger.info(snapshot)
        except IndexError:
            logger.info('No snapshot available')
            snapshot = None
        context[self.variable] = snapshot
        return ''

@register.tag(name='current_location')
def do_get_current_location(parser, token):
    tag, of_kwd, username, as_kwd, variable = token.split_contents()
    return LocationSnapshotNode(variable, username[1:-1])



import datetime
import json

from django.conf import settings
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.gis.geos import Point
from django.http import HttpResponse
from django.template.response import TemplateResponse
from django.views.decorators.csrf import csrf_exempt
from lxml import etree
import pytz
from social_auth.models import UserSocialAuth

from location import models

@staff_member_required
def my_location(request):
    try:
        most_recent_location = models.LocationSnapshot.objects.order_by('-date')[0]
    except IndexError:
        most_recent_location = None
    return TemplateResponse(
                request,
                'location/my_location.html',
                {
                    'location': most_recent_location
                }
            )

@staff_member_required
def get_kml(request):
    from pykml.factory import KML_ElementMaker as KML
    since = datetime.datetime.utcnow().replace(tzinfo=pytz.utc) - datetime.timedelta(days=int(request.GET.get('days', 7)))
    placemarks = []
    icon_styles = []
    coord_string = ""
    points = models.LocationSnapshot.objects.filter(
                date__gt=since
            ).select_related().order_by('date')
    source_types = models.LocationSourceType.objects.all()
    for source_type in source_types:
        if source_type.icon:
            icon_styles.append(
                        KML.IconStyle(
                                KML.scale(1.0),
                            ),
                        KML.Icon(
                                KML.href(settings.MEDIA_URL + source_type.icon.url)
                            ),
                        id="type%s" % source_type.id
                    )
    for point in points:
        coord_string = coord_string + str(point.location.coords[0]) + "," + str(point.location.coords[1]) + " "
        placemarks.append(
                    KML.Placemark(
                            KML.description("%s" % (point, )),
                            KML.styleUrl("#type%s" % point.source.type.id),
                            KML.Point(
                                    KML.coordinates(
                                            str(point.location.coords[0]) + "," + str(point.location.coords[1])
                                        )
                                )
                        )
                )
    style = KML.Style(
                *icon_styles,
                id='default'
            )
    path = KML.Placemark(
                KML.name("Path"),
                KML.LineString(
                        KML.coordinates(
                                coord_string
                            )
                    )
            )
    document = KML.kml(
                KML.Document(
                        KML.name('My recent path'),
                        * [style] + [path] + placemarks
                    )
            )
    response = HttpResponse(
            etree.tostring(document, pretty_print=True),
            mimetype='application/vnd.google-earth.kml+xml'
            )
    response['Content-Disposition'] = 'attachment; filename=%s.kml' % datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
    return response

@csrf_exempt
def foursquare_checkin(request):
    (source, created, ) = models.LocationSourceType.objects.get_or_create(name='Foursquare Check-in')
    raw_data = request.POST.get('checkin', None)
    data = json.loads(raw_data)
    if data['type'] == 'checkin':
        checkin = models.LocationSource()
        checkin.name = data['venue']['name']
        checkin.type = source
        checkin.data = raw_data
        checkin.save()

        socialauth = UserSocialAuth.objects.get(
                    uid=data['user']['id'],
                    provider='foursquare'
                )

        snapshot = models.LocationSnapshot()
        snapshot.user = socialauth.user
        snapshot.location = Point(
                    data['venue']['location']['lng'],
                    data['venue']['location']['lat'],
                )
        snapshot.source = checkin
        snapshot.save()
    return HttpResponse("OK")




# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'LocationSourceType'
        db.create_table('location_locationsourcetype', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('icon', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
        ))
        db.send_create_signal('location', ['LocationSourceType'])

        # Adding model 'LocationSource'
        db.create_table('location_locationsource', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['location.LocationSourceType'])),
            ('data', self.gf('jsonfield.fields.JSONField')()),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('location', ['LocationSource'])

        # Adding model 'LocationSnapshot'
        db.create_table('location_locationsnapshot', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('location', self.gf('django.contrib.gis.db.models.fields.PointField')(geography=True)),
            ('source', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='points', null=True, to=orm['location.LocationSource'])),
            ('date', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('location', ['LocationSnapshot'])


    def backwards(self, orm):
        # Deleting model 'LocationSourceType'
        db.delete_table('location_locationsourcetype')

        # Deleting model 'LocationSource'
        db.delete_table('location_locationsource')

        # Deleting model 'LocationSnapshot'
        db.delete_table('location_locationsnapshot')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'location.locationsnapshot': {
            'Meta': {'object_name': 'LocationSnapshot'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.contrib.gis.db.models.fields.PointField', [], {'geography': 'True'}),
            'source': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'points'", 'null': 'True', 'to': "orm['location.LocationSource']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'location.locationsource': {
            'Meta': {'object_name': 'LocationSource'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'data': ('jsonfield.fields.JSONField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['location.LocationSourceType']"}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'location.locationsourcetype': {
            'Meta': {'object_name': 'LocationSourceType'},
            'icon': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['location']


import datetime
import json
import logging
import time

from django.contrib.auth.models import User
from django.contrib.gis.geos import Point
from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import transaction
import pytz
import requests
from social_auth.models import UserSocialAuth

from location.models import LocationSourceType, LocationSource, LocationSnapshot

logger = logging.getLogger('location.management.commands.update_latitude_location')
logging.basicConfig(level=logging.INFO)

class Command(BaseCommand):
    args = '<django.contrib.auth.models.User.username>'
    help = 'Polls the Google Latitude API to gather the specified user\s last '\
            + 'known location.'

    @transaction.commit_on_success
    def handle(self, *args, **kwargs):
        user = User.objects.get(username = args[0])
        auth = UserSocialAuth.objects.get(
                    user = user,
                    provider = 'google-oauth2'
                )
        data = auth.extra_data

        if 'expiration_date' not in data.keys():
            data['expiration_date'] = 0
        if data['expiration_date'] < time.mktime(datetime.datetime.now().timetuple()):
            data = self.refresh_access_token(data)

        latitude_data = self.get_current_location(data)

        self.update_location(latitude_data, user)

        auth.extra_data = data
        auth.save()

    def update_location(self, data, user):
        (source_type, created) = LocationSourceType.objects.get_or_create(name='Google Latitude')
        local_tz = pytz.timezone('US/Pacific-New')
        date = datetime.datetime.fromtimestamp(
                    float(data['data']['timestampMs']) / 1000,
                    local_tz
                )

        existing_points = LocationSnapshot.objects.filter(
                    date__gt = date - datetime.timedelta(minutes=1),
                    source__type = source_type
                )
        if existing_points:
            logging.info("Our latest point is %s but I just found a point with date %s" % (
                    date,
                    local_tz.normalize(existing_points[0].date)
                    )
                )
            return
        logger.info("Point does not exist, updating.")
        source = LocationSource()
        source.name = "Google Latitude Location at  %s" % local_tz.normalize(date)
        source.type = source_type
        source.data = data
        source.active = False
        source.save()

        point = LocationSnapshot()
        point.user = user
        point.location = Point(
                    data['data']['longitude'],
                    data['data']['latitude'],
                )
        point.source = source
        point.date = date
        point.save()

    def refresh_access_token(self, data):
        r = requests.post(
                'https://accounts.google.com/o/oauth2/token',
                data={
                    'client_id': settings.GOOGLE_OAUTH2_CLIENT_ID,
                    'client_secret': settings.GOOGLE_OAUTH2_CLIENT_SECRET,
                    'refresh_token': data['refresh_token'],
                    'grant_type': 'refresh_token'
                    },
                )
        logger.info(r.text)
        response = json.loads(r.text)
        data['access_token'] = response['access_token']
        data['id_token'] = response['id_token']
        data['expiration_date'] = time.mktime(datetime.datetime.now().timetuple()) + response['expires_in']
        return data

    def get_current_location(self, data):
        r = requests.get(
                'https://www.googleapis.com/latitude/v1/currentLocation?granularity=best',
                headers={
                    'Authorization': 'Bearer ' + data['access_token']
                    }
                )
        logger.info(r.text)
        response = json.loads(r.text)
        return response




from setuptools import setup

setup(
    name='django-location',
    version='1.1',
    url='http://bitbucket.org/latestrevision/django-location/',
    description='Gather, store, and display real-time location information from Foursquare, Google Latitude, and more.',
    author='Adam Coddington',
    author_email='me@adamcoddington.net',
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering :: GIS',
    ],
    include_package_data=True,
    extras_require={
            'locationdetails': [
                'django-neighborhoods',
                'django-census-places',
            ],
            'runmeter': [
                'django-mailbox',
            ],
            'kml': [
                'pykml',
            ]
        },
    install_requires=[
        'django>=1.4',
        'django-social-auth',
        'pytz',
        'requests',
        'jsonfield',
        'lxml',
        ],
    packages=[
        'location',
        'location.management',
        'location.management.commands',
        'location.migrations',
        'location.templatetags',
        ],
)



import datetime
import logging
import urllib2
import re

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.gis.geos import Point
from django.core.management.base import BaseCommand
from django.db import transaction
from django.db.models import Max
from django.utils.timezone import utc
from lxml import objectify

from location.models import LocationSource, LocationSourceType, LocationSnapshot
from django_mailbox.models import Message

logger = logging.getLogger('location.management.commands.check_incoming_runmeter')
logging.basicConfig(level=logging.INFO)

MINIMUM_INTERVAL_SECONDS = getattr(settings, 'RUNMETER_MINIMAL_INTERVAL_SECONDS', 30)

class Command(BaseCommand):
    args = '<django_mailbox.models.Mailbox.name> <django.contrib.auth.models.User.username>' 
    help = 'Imports start and end e-mail messages sent by Runmeter (http://www.abvio.com/runmeter/) '\
            + 'delivered to a mailbox watched by django_mailbox, creating location points for the '\
            + 'username specified.'
    namespaces = {
            'kml': 'http://www.opengis.net/kml/2.2',
            'abvio': 'http://www.abvio.com/xmlschemas/1'
            }

    @transaction.commit_on_success
    def handle(self, *args, **options):
        self.user = User.objects.get(username=args[1])
        mailbox_name = args[0]
        messages = Message.unread_messages.filter(
                    mailbox__name = mailbox_name
                ).order_by('received')
        for message in messages:
            logger.info("Received message %s" % message)
            url = self.get_import_url(message)
            logger.debug("Import URL %s" % url)
            source = self.get_source(url)
            if self.is_finish_email(message):
                logger.info("Is finishing e-mail.")
                source.active = False
            self.process_source(source)
            read_date = datetime.datetime.utcnow().replace(utc)
            message.read = read_date
            message.save()
            logger.debug("Marked as read.")
        
        for ongoing_source in LocationSource.objects.filter(active=True):
            self.process_source(ongoing_source)

    def process_source(self, source):
        logger.info("Processing source %s" % source)
        document = self.get_document(source.data['url'])
        start_time = self.get_start_time(document)
        points = self.get_points(document)
        route_name = self.get_route_name(document)

        max_date = LocationSnapshot.objects\
                .filter(source=source)\
                .aggregate(max_date=Max('date'))['max_date']
        for key_name, data_point in points.items():
            if key_name not in source.data['known_points'].keys():
                logger.debug("%s not in %s" % (
                        key_name,
                        source.data['known_points'].keys()
                    ))
                logger.debug("Creating point %s,%s" % (
                        data_point['lat'],
                        data_point['lng'],
                    ))
                point_date = start_time + datetime.timedelta(
                            seconds = data_point['time']
                            )
                if max_date and max_date + datetime.timedelta(seconds=MINIMUM_INTERVAL_SECONDS) < point_date:
                    point = LocationSnapshot()
                    point.user = self.user
                    point.location = Point(
                        data_point['lat'],
                        data_point['lng']
                    )
                    point.source = source
                    point.date = point_date
                    point.save()
                    max_date = point_date
                source.data['known_points'][key_name] = data_point
            else:
                logger.debug("Point already exists")
        if route_name:
            source.name = "%s (%s)" % (
                    route_name if route_name else 'AdHoc',
                    source.data['url']
                    )
        if source.active:
            source.active = self.get_activity_status(document, source)
        source.save()

    def is_finish_email(self, message):
        if re.search('Finished [A-Za-z0-9-_]+:', message.body):
            return True
        return False

    def get_activity_status(self, document, source):
        max_date = source.points.aggregate(avg=Max('date'))['avg']
        now = datetime.datetime.utcnow().replace(tzinfo=utc)
        logger.debug("Max Date %s" % max_date)
        logger.debug("Now %s" % now)
        if max_date and now - max_date > datetime.timedelta(minutes = 60):
            return False
        return True

    def get_source(self, url):
        (source_type, created) = LocationSourceType.objects.get_or_create(name='Runmeter')
        source = None
        for potential_source in LocationSource.objects.filter(
                created__gt = datetime.datetime.now().replace(tzinfo=utc) - datetime.timedelta(days=1),
                active=True
                ):
            data = potential_source.data
            if 'url' in data.keys() and data['url'] == url:
                source = potential_source
                logger.info("Found existing source")
                break
        if not source:
            source = LocationSource()
            source.type = source_type
            source.name = "Runmeter Route at %s" % datetime.datetime.now()
            source.data = {
                        'url': url,
                        'known_points': {}
                        }
            source.active = True
            source.save()
            logger.info("Creating new source")
        return source

    def get_route_name(self, document):
        values = document.xpath('//abvio:routeName', namespaces=self.namespaces)
        if values:
            return values[0].text
        values = document.xpath('//abvio:activityName', namespaces=self.namespaces)
        if values:
            return values[0].text
        return None

    def get_start_time(self, document):
        start_string = document.xpath('//abvio:startTime', namespaces=self.namespaces)[0].text

        return datetime.datetime.strptime(start_string[0:19], "%Y-%m-%d %H:%M:%S").replace(
                tzinfo=utc
                )

    def get_points(self, document):
        points = {}
        coordinate_table = document.xpath('//abvio:coordinateTable', namespaces=self.namespaces)[0].text.split('\n')
        for coordinate_row in coordinate_table:
            if coordinate_row:
                cols = coordinate_row.split(',')
                points[coordinate_row] = {
                        'time': float(cols[0]),
                        'lng': float(cols[1]),
                        'lat': float(cols[2]),
                        }
        return points

    def get_document(self, url):
        return objectify.fromstring(
                urllib2.urlopen(url).read()
                )

    def get_import_url(self, message):
        matches = re.search(r"Import URL: (.*)", message.body)
        if not matches:
            return None
        return matches.groups()[0].strip()
